# coding: utf-8
import time
import logging

from celery import Celery, task

from django.contrib import admin
from django.http import HttpRequest
from django.utils.translation import ugettext_lazy as _

from jobtastic import JobtasticTask
from jmb.async.models import Task

logger = logging.getLogger('celery.task')


class JmbJobtasticTask(JobtasticTask):
    @classmethod
    def delay_or_fail(self, app_label=None, *args, **kwargs):
        logger.info("TASK 0: %s" % app_label)
        res =  super(JmbJobtasticTask, self).delay_or_fail(*args, **kwargs)
        task_id = res.task_id
        logger.info("TASK: %s" % (res.task_id))
        task_name = self.task_name 
        status = res.status
        jtask = Task(title=task_name, task_id=task_id, status=status, app_label=app_label)
        jtask.save()
        return jtask


class GenericJmbJobtasticTask(JmbJobtasticTask):
    """
    We don't want to cache the task, in that case Jobtastic would use the same results for different tasks
    (of the same type).
    We need to set herd_avoidance_time as 0 seconds and cache duration as a negative number.
    Doing that we don't enable caching for generic tasks::

        herd_avoidance_timeout: This is the max number of seconds for which Jobtastic will wait for identical task results to be determined
        cache_duration: This is the number of seconds for which identical jobs should try to just re-use the cached result.

    In other case we should determine which tasks are identical using significant_kwargs parameters.In fact that means::

        signficant_kwargs: It's a list of 2-tuples containing the name of a kwarg plus a function to turn that kwarg in to a string.
                           Jobtastic uses these to determine if your task should have an identical result to another task run
    """
    task_name = _('Generic Task')
    significant_kwargs = []
    herd_avoidance_timeout = 0
    cache_duration = -1

    def set_init_calculate_result(self, model, request_get, request_request, select_across, pks=(), **kwargs):
        self.success_count = 0
        self.error_count = 0
        self.success_msg = []
        self.error_msg = []
        self.kwargs = kwargs

        admin.autodiscover()
        model_admin = admin.site._registry[model]
        self.set_request(request_get=request_get, request_request=request_request)
        request = self.get_request()
        if select_across:
            self.queryset = model_admin.queryset(request)
        else:
            self.queryset = model_admin.model.objects.filter(id__in=pks)

    def calculate_result(
            self, app_label=None, model=None, request_get=None,
            request_request=None, select_across=None, pks=None,
             **kwargs
    ):
        self.set_init_calculate_result(model, request_get, request_request, select_across, pks, **kwargs)
        for count, qs_obj in enumerate(self.queryset):
            self.do_action(qs_obj)
            self.update_progress(
                count,
                self.queryset.count(),
                update_frequency=100,
            )

        self.set_messages()
        self.set_task()
        return self.task_name.capitalize()

    def set_request(self, request_get, request_request):
        self.request_get = request_get
        self.request_request = request_request

    def get_request(self):
        request = HttpRequest()
        request.GET = self.request_get
        request.REQUEST = self.request_request
        return request

    def get_messages_output(self):
        message = ""
        logger.info("message: %s - %s" % (self.success_msg, self.error_msg))
        if len(self.success_msg):
            message += "<br/>".join(self.success_msg)
        if len(self.error_msg):
            message += "<br/>".join(self.error_msg)
        return message

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("0 success found"),
            success_msg_1=_("1 success found"),
            success_msg_n=_("%s successes found" % self.success_count),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def set_messages(self):
        self.set_report_msg_dict()
        if self.success_count == 0:
            self.error_msg.insert(0, self.report_msg_dict['success_msg_0'])
        elif self.success_count == 1:
            self.success_msg.insert(0, self.report_msg_dict['success_msg_1'])
        elif self.success_count > 1:
            self.success_msg.insert(0, self.report_msg_dict['success_msg_n'])

        if self.error_count == 1:
            self.error_msg.insert(0, self.report_msg_dict['error_msg_1'])
        elif self.error_count > 1:
            self.error_msg.insert(0, self.report_msg_dict['error_msg_n'])

    def set_task(self):
        task_id = self.request.id
        task = Task.objects.get(task_id=task_id)
        task.message = self.get_messages_output()
        task.save()

    def do_action(self, obj):
        """
        You MUST define your personal action
        """
        raise NotImplementedError


class AspettaTask(JmbJobtasticTask):
    task_name = "Test task"
    significant_kwargs = []
    herd_avoidance_timeout = 60
    cache_duration = 0
    def calculate_result(self, **kw):
        logger.info('Running aspetta con xtastic ')

        n_of_steps = 20
        sec_for_step = 2
        # Only actually update the progress in the backend every n operations
        update_frequency = 1
        total_delay = 40

        for count, x in enumerate(range(0, total_delay, sec_for_step)):
            time.sleep(sec_for_step)
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=update_frequency,
            )
            logger.info('Running aspetta - %s' % x)
        return "Finito!"
